-- Livestock:  For summoning random mounts, flying mounts, and critters.

-- Concept:  Khanthal (Uldum)

-- Copyright (c) 2008 Scott Snowman (Recompense on Uldum(US)), scott.snowman@gmail.com
-- All rights reserved.

-- Namespace and saved variable tables
Livestock = {}
LivestockSettings = {}

-- Saved variable default values and version string
local defaults = {
	Critters = {},
	Mounts = {},
	showcritter = 0,
	showland = 0,
	showflying = 0,
	showsmart = 0,
	useslowland = 1,
	useslowflight = 1,
	scale = 1,
	druidlogic = 0,
	autosummon = 0,
	}
	
LIVESTOCK_VERSION = "1.0.2"

-- Localization tables

local L = LivestockLocalizations

	local stringstable = { -- strings to be localized in the GUI.  This table is indexed by the name of the fontstring with each value being the string that should be displayed.  This allows for a quick iteration to set the strings correctly.
	["LivestockMenuFrame3DLabel"] = L.LIVESTOCK_FONTSTRING_3DLABEL,
	["LivestockMacroFrameButtonsToggleTitle"] = L.LIVESTOCK_FONTSTRING_BUTTONSTOGGLETITLE,
	["LivestockMacroFrameMacroTitle"] = L.LIVESTOCK_FONTSTRING_MACROBUTTONTITLE,
	["LivestockMacroFrameOtherTitle"] = L.LIVESTOCK_FONTSTRING_OTHERTITLE,
	["LivestockMacroFrameShowCrittersLabel"] = L.LIVESTOCK_FONTSTRING_SHOWCRITTERSLABEL,
	["LivestockMacroFrameShowLandMountsLabel"] = L.LIVESTOCK_FONTSTRING_SHOWLANDLABEL,
	["LivestockMacroFrameShowFlyingMountsLabel"] = L.LIVESTOCK_FONTSTRING_SHOWFLYINGLABEL,
	["LivestockMacroFrameShowSmartMountsLabel"] = L.LIVESTOCK_FONTSTRING_SHOWSMARTLABEL,
	["LivestockMacroFrameMacroText"] = L.LIVESTOCK_FONTSTRING_MACROBUTTONSTITLE,
	["LivestockMacroFrameUseSlowLandText"] = L.LIVESTOCK_FONTSTRING_USESLOWLANDTEXT,
	["LivestockMacroFrameUseSlowFlyingText"] = L.LIVESTOCK_FONTSTRING_USESLOWFLYINGTEXT,
	["LivestockMacroFrameAutoSummonOnRezText"] = L.LIVESTOCK_FONTSTRING_AUTOSUMMONONREZ,
	["LivestockMacroFrameDruidToggleText"] = L.LIVESTOCK_FONTSTRING_DRUIDTOGGLETEXT,
	["LivestockCritterMenuButton"] = L.LIVESTOCK_FONTSTRING_SHOWCRITTERSLABEL,
	["LivestockLandMountMenuButton"] = L.LIVESTOCK_FONTSTRING_SHOWLANDLABEL,
	["LivestockFlyingMountMenuButton"] = L.LIVESTOCK_FONTSTRING_SHOWFLYINGLABEL,
	["LivestockCritterMacroButton"] = L.LIVESTOCK_FONTSTRING_SHOWCRITTERSLABEL,
	["LivestockLandMountMacroButton"] = L.LIVESTOCK_FONTSTRING_SHOWLANDLABEL,
	["LivestockFlyingMountMacroButton"] = L.LIVESTOCK_FONTSTRING_SHOWFLYINGLABEL,
	["LivestockComboMacroButton"] = L.LIVESTOCK_FONTSTRING_SHOWSMARTLABEL,
	}

-- Variable declaration
BINDING_HEADER_LIVESTOCKHEADER = "Livestock Summons" -- Keybindings menu
_G["BINDING_NAME_CLICK LivestockComboButton:LeftButton"] = "Random Smart Mount" -- Keybinding for the Smart Mount binding

local currentcritter, eastereggtext, hasflying, menusbuilt, pickland, pickflying, mountinpostclick, oktosummon
local modelcounter = 0
local NUMBER_OF_CRITTER_MENUS, NUMBER_OF_LAND_MENUS, NUMBER_OF_FLYING_MENUS = 1, 1, 1  -- constants for menu appearance and building
local crittersText, landText, flyingText, mountstable, temp = {}, {}, {}, {}, {} -- tables that will be reused
local buildtable = { -- this table contains information that we will use for building and rebuilding menus as well as hiding menus
	["CRITTER"] = {token = "Critter", text = crittersText, settings = LivestockSettings.Critters},
	["LAND"] = {token = "Land", text = landText, settings = LivestockSettings.Mounts},
	["FLYING"] = { token = "Flying", text = flyingText, settings = LivestockSettings.Mounts},
	}

local sayings = {
	"Please... no more...",
	"Can I get a new master?",
	"Why did I sign up for this gig?",
	"*HRRRRKKK*",
	"Only you can prevent animal vomit!",
	"We are not amused.",
	"Great job, spin doctor.",
	"You spin me right round, baby right round...",
	"PLEASE put the button back in the box!!",
	"So... dizzy...",
	"You expecting me to take off or something?",
	"Animal cruelty detected.\nContacting PETA... please hold.",
	"Does your mother know you treat\nanimals like this?",
	"Don't you have quests to do or something?",
	"I think you're a bit confused.  He said\n'spell rotation' NOT 'pet rotation!'",
	"Stop this thing, I want a refund!",
	"When I can see straight again, I'm\nreporting you for being mean.",
	"OK, OK!  I'm sorry I peed in your backpack!",
	}

CreateFrame("GameTooltip","LivestockTooltip",UIParent,"GameTooltipTemplate") -- Build our own tooltip to use for scanning.

local function p(arg)  -- local print function for Debugging (this was before I figured out that there was a native WoW print() function introduced around the time I wrote this mod
	DEFAULT_CHAT_FRAME:AddMessage(arg)
end

-- Function list:
	-- Event handlers:		Livestock.OnEvent(self, event, arg1): handles variable initialization (ADDON_LOADED)
	--				Livestock.CompanionEvent(self, event, ...):  iterates over the companions again, updating SV as needed and rebuilding menus at changes to the data.  Also sets the behavior for the smart mount functionality,  Used at login (PLAYER_LOGIN), when learning a new companion (COMPANION_LEARNED), and when changing areas (ZONE_CHANGED_NEW_AREA).
	-- Menu creation:		Livestock.BuildMenu(type):  determines the number of dropdown menus needed based on the type of mount given by "type", calculates their size, and builds and parents them appropriately.
	--				Livestock.BuildMenus(): three calls to BuildMenu to create the three dropdown menus
	--				Livestock.RebuildMenu(type):  hides the entirety of dropdown menu "type" and then calls BuildMenu(type) to redraw the menu.  Used to visually update the menu when companion data changes.
	-- GUI functions:		Livestock.RestoreUI():  Used on startup to manage the UI elements and check/uncheck, hide/show them as necessary.
	--				Livestock.HideDropdowns():  hides all visible dropdown menus.  Used when opening the macro frame or clicking a menu button to make sure old menus aren't overlapping new ones.
	--				Livestock.HeaderButtonOnClick():  OnClick handlers for the menu buttons.  Hides dropdowns and shows the relevant dropdown if it wasn't visible when the button was clicked.
	--				Livestock.MenuButtonOnClick(self): retrieves data associated with self to determine what to do with the button when clicked.  Toggles pets if it's a pet button, checks/unchecks all if it's a check/uncheck all button
	--				Livestock.MenuButtonOnEnter(self):  retireves data associated with self to determine what to do with the button when the mouse is over it.  Sets the 3D model to the mousedover pet if it's a pet.  Displays more submenus if it's a submenu button.
	--				Livestock.SpinModel(self, elapsed):  pulls the current value from the spin speed slider and determines the speed at which to spin the model, then rotates the model.
	--				Livestock.SpinSliderChanged(self): checks the current value of the spin slider to see if the companion should be complaining or not ;)
	--				Livestock.ScaleButtons(value): scales the Livestock buttons according to the current value of the scale slider
	-- Data functions:		Livestock.RecycleData(table): goes through table and sets all keys to nil to reuse the table
	--				Livestock.RestoreDefaults(): syncs the saved variables to match the structure of the defaults.  Keys in the defaults not in the SV will be added from the defaults.  Keys in the SV not in the defaults will be set to nil.
	--				Livestock.RenumberCompanions():  Iterates over companions to check that the saved variables index matches the current index.  Makes new entries in SV for new companions, including parsing the tooltip to determine mount type and speed.  Has some debug code commented out because GetCompanionInfo() sometimes returns a nil name.
	--				Livestock.SmartPreClick(self):  The PreClick handler for the smart mount button.  Determines whether or not the button needs to perform a secure action (canceling Ghost Wolf or Travel Form to mount up, or casting a Flight Form).  Also determines if the area is flyable or not, and flags the PostClick handler to fire properly if a mount needs to be summoned.
	--				Livestock.SmartPostClick(self):  The PostClick handler mounts the character if it's supposed to, based on the land/flying flag set in the PreClick handler (mounting is not a secure function so it can be done this way)
	--				Livestock.MakeCombatSmartMount(self, class):  Called only with class == "DRUID" or "SHAMAN" and redefines the attributes of the SmartMount button to cast Ghost Wolf or Travel Form so that these can be accessed quickly in combat.
	-- Summon functions:	Livestock.PickCritter():  Randomly summons a critter from the list of critters the user has defined.  Will summon a different critter than the one currently out if more than one critter is selected.  Saves this critter to local memory (currentcritter)
	--				Livestock.DismissCritter():  Dismisses the current critter.  Tries to do it quickly by accessing the local currentcritter, but if it can't, will iterate over all critters to find the active one and dismiss it.
	--				Livestock.PickLandMount():  Randomly summons a land mount from the list of land mounts the user has defined.
	--				Livestock.PickFlyingMount():  Randomly summons a flying mount from the list of land mounts the user has defined.
	--Slash handler:		Livestock.Slash(arg): Resets Livestock buttons to default, scales the buttons, or toggles the menu bar.  Also toggles druid logic.
	
--Event handlers:
	
function Livestock.OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" then
		Livestock.RestoreDefaults() -- syncs the saved variables to match the structure of the defaults.  Keys in the defaults not in the SV will be added from the defaults.  Keys in the SV not in the defaults will be set to nil.
	end
end

function Livestock.CompanionEvent(self, event, ...)
	if event == "PLAYER_LOGIN" then -- when the player logs in, make sure the map is set right for Smart Detection and sort out the companions
		SetMapToCurrentZone()
		Livestock.RenumberCompanions()
	elseif event == "COMPANION_LEARNED" then -- when a new companion is learned, sort out the companions again and then rebuild the three menus
		Livestock.RenumberCompanions()
		Livestock.RebuildMenu("CRITTER")
		Livestock.RebuildMenu("LAND")
		Livestock.RebuildMenu("FLYING")
	elseif event == "PLAYER_REGEN_DISABLED" then -- if we got into combat, the SmartButton needs to become a Travel Form or Ghost Wolf button if we're a druid or Shaman
		local class = select(2, UnitClass("player"))
		if class == "DRUID" or class == "SHAMAN" then
			Livestock.MakeCombatSmartMount(LivestockComboButton, class) -- change the button
		end
	elseif strsub(event, 1, 12) == "ZONE_CHANGED" then -- shortcut to get at ZONE_CHANGED, ZONE_CHANGED_INDOORS, ZONE_CHANGED_NEW_AREA -- set the map to the current zone just in case
		SetMapToCurrentZone()
	elseif event == "PLAYER_UNGHOST" or event == "PLAYER_ALIVE" then -- summon a critter automatically upon death if the selected option is true
		if ( LivestockSettings.autosummon == 1 and not UnitIsDeadOrGhost("player") and oktosummon ) then
			Livestock.PickCritter()
		end
	end
end

-- Menu creation:

function Livestock.BuildMenu(kind) -- type is a string, either "CRITTER", "LAND", or "FLYING"
	local token, texttable = buildtable[kind].token, buildtable[kind].text -- buildtable contains references and variables that are specific to each type of menu, indexed by the same string as the variable type
	local mounts, MENUS, settings
	
	Livestock.RecycleTable(mountstable)
	Livestock.RecycleTable(texttable)
	
	if kind == "CRITTER" then
		settings = LivestockSettings.Critters
		NUMBER_OF_CRITTER_MENUS = math.ceil( GetNumCompanions(kind) / 20 ) -- break the number of critters into groups of 20
		MENUS = NUMBER_OF_CRITTER_MENUS -- make the local MENUS equal to the constant
		
	elseif kind == "LAND" then
		settings = LivestockSettings.Mounts
		mounts = 0 -- reset mounts to 0 so we can start counting land mounts
		for k in pairs(settings) do -- Check the list of mounts, pull out fast land mounts (or slow ones if the filter lets them through), increment mounts, and add the index of the mounts to mountstable
			if settings[k].type == "land" and (settings[k].speed == "epic" or LivestockSettings.useslowland == 1) then
				mounts = mounts + 1
				tinsert(mountstable, settings[k].index)
			end
			table.sort(mountstable) -- the table indices are all jumbled because of the iteration over settings, so this puts them in numerical order (guaranteeing the menu is in alphabetical order)
		end
		NUMBER_OF_LAND_MENUS = math.ceil( mounts / 20 ) -- break land mounts into groups of 20
		MENUS = NUMBER_OF_LAND_MENUS -- same as above

	elseif kind == "FLYING" then -- code is the same for flying mounts with the appropriate changes; perhaps this could be a function call?
		settings = LivestockSettings.Mounts
		mounts = 0
		for k in pairs(settings) do
			if settings[k].type == "flying" and (settings[k].speed == "epic" or LivestockSettings.useslowflight == 1) then
				mounts = mounts + 1
				tinsert(mountstable, settings[k].index)
			end
			table.sort(mountstable)
		end
		NUMBER_OF_FLYING_MENUS = math.ceil( mounts / 20)
		MENUS = NUMBER_OF_FLYING_MENUS --  same as above
	end
	
	
	for currentMenu = 1, MENUS do -- iterate over however many menus we're going to need
	
		local menuframe = _G["Livestock"..token.."Menu"..currentMenu] or CreateFrame("Frame","Livestock"..token.."Menu"..currentMenu, LivestockMenuFrame, "LivestockBlueFrameTemplate") -- make the background of the menu (or reference it if it already exists), naming it with the proper token and number
		menuframe:SetID(currentMenu) -- associate the number of the menu with the frame in a way that's unrelated to the name
		
		if currentMenu == 1 then -- if this is the first menu in a group, we need to parent it to the button that called it
			local header
			if kind == "CRITTER" then
				header = "LivestockCritterMenuButton"
			elseif kind == "LAND" then
				header = "LivestockLandMountMenuButton"
			elseif kind == "FLYING" then
				header = "LivestockFlyingMountMenuButton"
			end
			menuframe:SetPoint("TOPLEFT", header ,"BOTTOMLEFT",0,-10) -- set the first menu to appear below the associated button
		else
			local lastmenu = "Livestock"..token.."Menu"..(currentMenu-1)
			menuframe:SetPoint("TOPLEFT",lastmenu,"TOPRIGHT",3,0)  -- set other menus to appear to the right of the previous menu
		end
		
		if currentMenu == 1 and MENUS > 1 then  -- if there's more than 1 menu and this is the first menu, we'll have 20 animals, 1 "More", and Check / Uncheck All for 23 blocks total - about 435 high
			menuframe:SetHeight(435)
		elseif currentMenu ~= MENUS then -- the middle menus need 21 blocks total. about 400 high (20 animals, 1 More)
			menuframe:SetHeight(400)
		elseif MENUS > 1 then -- the last menu , which doesn't have check / uncheck all, depends on the number of companions
			menuframe:SetHeight( ( (mounts or GetNumCompanions("CRITTER")) % 20 * 18 ) + 21 )
		else -- there's only one menu, so no "More", but check and uncheck are needed
			menuframe:SetHeight( ( ( (mounts or GetNumCompanions("CRITTER")) % 20 + 2) * 18) + 21)
		end
		
		local endButton -- this is the number of buttons in the menu...
		
		if currentMenu ~= MENUS then
			endButton = 20 -- 20 if it's not the last menu
		else
			endButton = (mounts or GetNumCompanions("CRITTER")) % 20 -- last menu will be the remainder from the other groups of 20
		end
			
		local maxWidth = 100 -- default width of the buttons
			
		for button = 1, endButton do -- iterate over the buttons we need
			local menubutton = _G["Livestock"..token.."Menu"..currentMenu.."Button"..button] or CreateFrame("Button", "Livestock"..token.."Menu"..currentMenu.."Button"..button, menuframe, "LivestockMenuButtonTemplate") -- create or reference the button, with the appropriate name and number
			menubutton:Show()
			local text = _G["Livestock"..token.."Menu"..currentMenu.."Button"..button.."Text"] -- reference the text of the button
			
			local id, name
			if kind == "CRITTER" then
				id, name = GetCompanionInfo("CRITTER", (currentMenu - 1) * 20 + button )
			elseif kind == "LAND" or kind == "FLYING" then
				id, name = GetCompanionInfo("MOUNT", mountstable[(currentMenu - 1) * 20 + button] )
			end -- get the ID and name of the companion for the button by mapping the position of the button in the menus(s) to the index of the companion

			menubutton.critterID = id 
			menubutton.which = "toggle"..kind
			menubutton.name = name  -- associate the info for the companion with the frame itself to be used for mousing over / clicking
			
			text:SetText( name )
			tinsert(texttable, text) -- set the text of the button to reflect the name of the companion, and store the fontstring itself in a table of fontstrings
			
			if name and settings[name].show == 1 then -- if the name isn't nil (Blizzard bug sometimes returns nil for companion names >:o) and the settings say this companion is selected then
				text:SetTextColor(1, 1, 1) -- set the color of the text to be white
			elseif name then
				text:SetTextColor(0.4, 0.4, 0.4) -- otherwise set the color to be grey
			elseif not name then -- debug message if the name is nil
				local which, index
				if mounts then 
					which = "MOUNT"
					index = mountstable[(currentMenu - 1) * 20 + button]
				else
					which = "CRITTER"
					index = (currentMenu - 1 ) * 20 + button 
				end
				
				--[[local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo(which, index)
				DEFAULT_CHAT_FRAME:AddMessage("Max "..which.." is " .. (GetNumCompanions(which) or "nil"))
				DEFAULT_CHAT_FRAME:AddMessage(string.format("Error for "..which.." %u at menu build, ID: %s, name: %s, spellID: %s, icon: %s, summoned: %s", index, (creatureID or "nil"), (creatureName or "nil"),(creatureSpellID or "nil"), (icon or "nil"), (issummoned or "nil")))]]
			end
			
			menubutton:SetHeight(18) -- give the button a height
			if button == 1 then -- parent it to the menu's TOPLEFT corner if it's the first button, else underneath the previous button
				menubutton:SetPoint("TOPLEFT", menuframe, "TOPLEFT", 20, -10)
			else
				menubutton:SetPoint("TOPLEFT", _G[menuframe:GetName().."Button"..(button - 1)], "BOTTOMLEFT")
			end

			if button == endButton then -- set all the buttons to have the same width as the widest button.  Do this by iterating over the buttons, getting the width of their text, and then setting the buttons' width to be the width of the text plus a cushion of 50
				for i = 1, endButton do
					local textwidth = _G["Livestock"..token.."Menu"..currentMenu.."Button"..i.."Text"]:GetWidth()
					if textwidth > maxWidth then
						maxWidth = textwidth + 50
					end
				end
				for i = 1, endButton do
					_G["Livestock"..token.."Menu"..currentMenu.."Button"..i]:SetWidth(maxWidth)
				end
			end
			menuframe:SetWidth(maxWidth + 30) -- bump up the size of the menu frame background to compensate and make sure the border and background look nice
		end
		
		if currentMenu ~= MENUS then -- if we're not at the last menu, we need a "More" button
			local moreframe = _G["Livestock"..token.."Menu"..currentMenu.."MoreButton"] or CreateFrame("Button", "Livestock"..token.."Menu"..currentMenu.."MoreButton", _G["Livestock"..token.."Menu"..currentMenu], "LivestockMenuButtonTemplate") --  reference or create the button
			moreframe:SetHeight(18)
			moreframe:SetWidth(maxWidth) -- height and width match those of the other buttons
			local t = _G[moreframe:GetName().."Text"]
			t:SetText(L.LIVESTOCK_MENU_MORE)
			moreframe:SetPoint("TOPLEFT", "Livestock"..token.."Menu"..currentMenu.."Button"..endButton, "BOTTOMLEFT") -- stick it underneath the last button in the menu
			moreframe.which = "more"..kind -- associate a string with the frame to use for mouseover
		end
		
		if currentMenu == 1 then -- the first menu needs "Select all / none" buttons
			local check = _G["Livestock"..token.."MenuCheckButton"] or CreateFrame("Button", "Livestock"..token.."MenuCheckButton", _G["Livestock"..token.."Menu1"], "LivestockMenuButtonTemplate")
			check.which = "check"..kind
			local uncheck = _G["Livestock"..token.."MenuUncheckButton"] or CreateFrame("Button", "Livestock"..token.."MenuUncheckButton", _G["Livestock"..token.."Menu1"], "LivestockMenuButtonTemplate")
			uncheck.which = "uncheck"..kind
			check:SetHeight(18)
			check:SetWidth(maxWidth)
			uncheck:SetHeight(18)
			uncheck:SetWidth(maxWidth) -- reference them or make them, show them, set their heights and widths, associate info with the frame, etc.
			_G[check:GetName().."Text"]:SetText(L.LIVESTOCK_MENU_SELECTALL)
			_G[check:GetName().."Text"]:SetTextColor(1, 1, 0)
			_G[uncheck:GetName().."Text"]:SetText(L.LIVESTOCK_MENU_SELECTNONE)
			_G[uncheck:GetName().."Text"]:SetTextColor(1, 1, 0) -- set their text and color (yellow to stand out)
			if _G["Livestock"..token.."Menu1MoreButton"] then -- if the first menu has a More button, stick the Check All button under the More button.  Otherwise, stick it under the last menu button.
				check:SetPoint("TOPLEFT", _G["Livestock"..token.."Menu1MoreButton"], "BOTTOMLEFT")
			else
				check:SetPoint("TOPLEFT", _G["Livestock"..token.."Menu1Button"..endButton], "BOTTOMLEFT")
			end
			uncheck:SetPoint("TOPLEFT", check, "BOTTOMLEFT") -- stick the Uncheck All button under the Check All button... and we're done!
		end
	end
end

function Livestock.BuildMenus() -- call the build menu function on all three menus to initialize them
	Livestock.BuildMenu("CRITTER")
	Livestock.BuildMenu("LAND")
	Livestock.BuildMenu("FLYING")
	menusbuilt = true
end

function Livestock.RebuildMenu(kind) -- the menus change when you learn new companions, so we need to rebuild them occasionally.  kind is a string as before, either "CRITTER", "LAND", or "MOUNT"
 
	for i = 1, 8 do  -- iterate over 8 menus, which is definitely more than we need right now
		for j = 1, 20 do -- it's easiest just to assume 20 animals in each menu and break when there aren't than to see how many animals there actually are
			if _G["Livestock"..buildtable[kind].token.."Menu"..i.."Button"..j] then -- if the button exists, then hide it
				_G["Livestock"..buildtable[kind].token.."Menu"..i.."Button"..j]:Hide()
			else -- if it doesn't exist, we are out of animals and should stop looping
				break
			end
		end
		if _G["Livestock"..buildtable[kind].token.."Menu"..i] then
			_G["Livestock"..buildtable[kind].token.."Menu"..i]:Hide() -- once all the buttons are hidden, check to see if the menu exists.  if it does, hide it as well.  If not, stop looping.
		else
			break
		end
	end
	
	Livestock.BuildMenu(kind) -- call BuildMenu to create the new button, adjust the old ones, resize things if needed, create a new menu if needed, etc.
	
end

-- GUI functions:

function Livestock.RestoreUI()
-- UI elements in tables to make it easier to set up.  It's not good to declare these inside a function, but the variables aren't available the right way until after the UI loads so this function needs to have the tables declared here.
	local buttonstable = {
	["LivestockSmartButton"] = LivestockSettings.showsmart,
	["LivestockCrittersButton"] = LivestockSettings.showcritter,
	["LivestockLandMountsButton"] = LivestockSettings.showland,
	["LivestockFlyingMountsButton"] = LivestockSettings.showflying,
	}

	local checkstable = {
	["LivestockMacroFrameShowSmartMounts"] = LivestockSettings.showsmart,
	["LivestockMacroFrameShowCritters"] = LivestockSettings.showcritter,
	["LivestockMacroFrameShowLandMounts"] = LivestockSettings.showland,
	["LivestockMacroFrameShowFlyingMounts"] = LivestockSettings.showflying,
	["LivestockMacroFrameShowSlowLandMounts"] = LivestockSettings.useslowland,
	["LivestockMacroFrameShowSlowFlyingMounts"] = LivestockSettings.useslowflight,
	["LivestockMacroFrameToggleAutosummon"] = LivestockSettings.autosummon,
	["LivestockMacroFrameToggleDruidLogic"] = LivestockSettings.druidlogic,
	}
	
	for k, v in pairs(buttonstable) do -- show and scale each of the buttons if its setting is saved as 1 (shown)
		if v == 1 then
			_G[k]:Show()
			_G[k]:SetScale(LivestockSettings.scale)
		end
	end
	
	for k, v in pairs(checkstable) do -- check each box if its setting is saved as 1 (checked)
		if v == 1 then
			_G[k]:SetChecked(true)
		end
	end
	
	for k, v in pairs(stringstable) do -- localize the fontstrings in the GUI
		_G[k]:SetText(v)
	end
	
	if select(2,UnitClass("player")) ~= "DRUID" then -- Gray out the text for the druid-only option if the player isn't a druid.  The disabling of the button is done in the button's OnLoad script.
		LivestockMacroFrameDruidToggleText:SetTextColor(0.4, 0.4, 0.4)
	end
	
	oktosummon = true
	
end

function Livestock.HideDropdowns() --  hide all the visible dropdown menus
	for k in pairs(buildtable) do
		for j = 1, 8 do -- the number of menus may change per type, but it's certainly not more than 8
			if _G["Livestock"..buildtable[k].token.."Menu"..j] and _G["Livestock"..buildtable[k].token.."Menu"..j]:IsVisible() then
				_G["Livestock"..buildtable[k].token.."Menu"..j]:Hide()
			end
		end
	end
end

function Livestock.HeaderButtonOnClick(token) -- when we click a main menu button, we should hide all dropdowns (so they don't overlap) and show the relevant menu, but only if it wasn't visible in the first place.  This lets the buttons act as true toggles.
	if not menusbuilt then
		Livestock.BuildMenus()
	end
	
	local menu, visible = _G["Livestock"..token.."Menu1"]
	if menu and menu:IsVisible() then
		visible = true
	end
	Livestock.HideDropdowns()
	if menu and not visible then
		menu:Show()
	end
end

function Livestock.MenuButtonOnClick(self) -- self is the button clicked on

	local action, companionType = strmatch(self.which, "(%l+)"), strmatch(self.which, "(%u+)") -- since the buttons come with a string attached that is of the format "actionTYPE" where action is the function of the button and TYPE is the capitalized animal type, this declaration breaks them apart into two again
	local settings, checkwhich
	
	if companionType == "CRITTER" then
		settings = LivestockSettings.Critters
	else
		settings = LivestockSettings.Mounts
		if companionType == "LAND" then
			checkwhich = LivestockSettings.useslowland
		elseif companionType == "FLYING" then
			checkwhich = LivestockSettings.useslowflight
		end
	end
	
	if action == "toggle" then -- if this is a toggle type button then
	
		local name, text = self.name, _G[self:GetName().."Text"] -- name was associated with the frame to be the name of the animal inside, and text is a reference to the text string of the button
		settings[name].show = 1 - settings[name].show -- toggle the shown setting
		if settings[name].show == 1 then
			text:SetTextColor(1, 1, 1)
		else
			text:SetTextColor(0.4, 0.4, 0.4) --  color the fontstring according to the new setting
		end
		
	elseif action == "check" then -- if this is a "check all" button
		
		for k in pairs(settings) do -- if these are critters or if the mounts in the settings have the same type as the companionType, lowercased (either "land" or "flying") then toggle them on
			if companionType == "CRITTER" or settings[k].type == companionType:lower() then
				if companionType == "CRITTER" or (settings[k].speed == "epic") or (settings[k].speed == "slow" and checkwhich == 1) then
					settings[k].show = 1
				end
			end
		end
		for _, v in ipairs(buildtable[companionType].text) do -- iterate over the fontstrings and set them all to white
			v:SetTextColor(1, 1, 1)
		end
		
	elseif action == "uncheck" then --  otherwise it should be an "uncheck all" button
		
		for k in pairs(settings) do -- if these or critters or if the mounts in the settings have the same type as the companionType, lowercased (either "land" or "flying") then toggle them on
			if companionType == "CRITTER" or settings[k].type == companionType:lower() then
				settings[k].show = 0
			end
		end
		for _, v in ipairs(buildtable[companionType].text) do -- iterate over the fontstrings and set them all to grey
			v:SetTextColor(0.4, 0.4, 0.4)
		end
	end
end

function Livestock.MenuButtonOnEnter(self) -- code for mousing over a menu button.  self is the menu button
	local which = self.which --  "which" was a string associated with the button
	
	if strsub(which, 1, 6) == "toggle" then -- if this is a toggle button, when we mouseover, we should show the 3D model if the viewer is visible
		local id = self.critterID -- the id of the creature was associated with the button when the menu was built
		if LivestockModel:IsVisible() then
			LivestockModel:SetCreature(id)
			LivestockModel.angle = 0.71
		end
	
	elseif strsub(which, 1, 4) == "more" then -- if this is a more button, mousing over needs to show the next menu
		_G["Livestock"..buildtable[strsub(which, 5, strlen(which))].token.."Menu"..(self:GetParent():GetID() + 1)]:Show() -- find the menu associated with the button by getting its parent, adding 1 to the number of the menu, and then showing the resulting menu.  The table reference looks up the right token for the name of the menu based on the end of the "which" string
	end
end

function Livestock.SpinModel(self, elapsed) -- this is the OnUpdate script for the model
	local angle = self.angle or 0.71
	modelcounter = modelcounter + elapsed
	if modelcounter > 0.05 then
		angle = angle + LivestockSpinSlider:GetValue()
		self:SetFacing(angle)
		modelcounter = 0
	end
	self.angle = angle
end  -- while the frame is visible, this code uses a counter to keep track of the total time the frame was last redrawn.  Every 0.05 sec, the model rotates according to the value established by the slider in the viewer

function Livestock.SpinSliderChanged(self) -- easter egg text to show up when you drag the 3D model spin slider so far over it "escapes" the box
	if self:GetValue() > 0.623 and (not eastereggtext) then -- if the slider is dragged above the appropriate value and there's no easteregg text shown
		Livestock.HideDropdowns() -- hide the menus or else the cursor ends up going into one of them which changes the model
		eastereggtext = true
		LivestockEasterEgg:Show()
		LivestockEasterEgg:SetText(sayings[random(#sayings)])
	elseif self:GetValue() < 0.623 then
		LivestockEasterEgg:Hide()
		eastereggtext = nil
	end
end

function Livestock.ScaleButtons(value) -- scale the buttons and reset them to their default positions
	if not LivestockCrittersButton then -- this script fires before the buttons are set, so return early if that's the case
		return
	end
	LivestockSettings.scale = value
	LivestockCrittersButton:SetScale(value)
	LivestockCrittersButton:ClearAllPoints()
	LivestockCrittersButton:SetPoint("CENTER",-50,-150)
	LivestockLandMountsButton:ClearAllPoints()
	LivestockLandMountsButton:SetScale(value)
	LivestockLandMountsButton:SetPoint("CENTER",0,-150)
	LivestockFlyingMountsButton:ClearAllPoints()
	LivestockFlyingMountsButton:SetScale(value)
	LivestockFlyingMountsButton:SetPoint("CENTER",50,-150)
	LivestockSmartButton:ClearAllPoints()
	LivestockSmartButton:SetScale(value)
	LivestockSmartButton:SetPoint("CENTER",0,-190)
end

-- Data functions

function Livestock.RecycleTable(table)
	for k in pairs(table) do
		table[k] = nil
	end
end

function Livestock.RestoreDefaults()
	if arg1 == "Livestock" then
		for k, v in pairs(defaults) do
			if not LivestockSettings[k] then
				LivestockSettings[k] = v
			end
		end
		for k, v in pairs(LivestockSettings) do
			if not defaults[k] then
				LivestockSettings[k] = nil
			end
		end
	end
end

function Livestock.RenumberCompanions() -- because the game does not distinguish between learning a new mount or a new critter, it's safest just to redo both indices

	for i = 1, GetNumCompanions("CRITTER") do
		local _,name = GetCompanionInfo("CRITTER",i)
		if name and not LivestockSettings.Critters[name] then -- sometimes Blizzard's code returns nil for the name.  If we have a valid name but no associated data in the SV, then we need to add it.
			LivestockSettings.Critters[name] = { -- save the index and set the default show value to on
				show = 1,
				index = i,
			}
		elseif name then -- if there is a valid name, then make sure the index matches up because indices change when you learn a new companion
			LivestockSettings.Critters[name].index = i
		else --  debug message if the name is returned as nil
			critterfail = true
			--[[local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("CRITTER", i)
			DEFAULT_CHAT_FRAME:AddMessage("Max pets is " .. (GetNumCompanions("CRITTER") or "nil"))
			DEFAULT_CHAT_FRAME:AddMessage(string.format("Error for critter %u at redo companions, ID: %s, name: %s, spellID: %s, icon: %s, summoned: %s", i, (creatureID or "nil"), (creatureName or "nil"),(creatureSpellID or "nil"), (icon or "nil"), (issummoned or "nil")))]]
		end
	end
	
	for i = 1, GetNumCompanions("MOUNT") do
		local _, name, spellID = GetCompanionInfo("MOUNT",i) -- we need not only the name but also the spellID because we will parse this spell's tooltip
		GameTooltip_SetDefaultAnchor(LivestockTooltip, UIParent) 
		LivestockTooltip:SetHyperlink("spell:"..spellID) -- pull up the tooltip object we created and set it to the ID of the spell
		if name and not LivestockSettings.Mounts[name] then -- if the name is valid and we don't have SV data for it, we need to make it
			
			local outland, northrend, very, fast
			
			for i = 1, LivestockTooltip:NumLines() do -- iterate over the number of lines in the tooltip
				local text = _G["LivestockTooltipTextLeft"..i]:GetText() -- get the text from each line
				for word in string.gmatch(text, "%a+") do -- look at each word in the line
					if word == "Outland" then
						outland = true
					elseif word == "Northrend" then
						northrend = true
					elseif word == "very" then
						very = true
					elseif word == "fast" then
						fast = true
					end
				end
			end -- parse the tooltip.  The variables outland, northrend, very, and fast are set to true if the corresponding text is found
			
			LivestockSettings.Mounts[name] = { index = i, } -- set the index of the mount
			
			if northrend and outland then -- if the tooltip contains both of these words, it's a flying mount
				LivestockSettings.Mounts[name].type = "flying"
			else -- otherwise, a land mount
				LivestockSettings.Mounts[name].type = "land"
			end
			
			if very and fast then -- if the tooltip contains both of these words, it's an epic mount of some sort. 
				LivestockSettings.Mounts[name].speed = "epic"
				LivestockSettings.Mounts[name].show = 1 -- Epic mounts always get shown by default.
			else
				LivestockSettings.Mounts[name].speed = "slow" -- otherwise it's a slow mount of some kind.
				if (LivestockSettings.Mounts[name].type == "flying" and LivestockSettings.useslowflight == 1) or (LivestockSettings.Mounts[name].type == "land" and LivestockSettings.useslowland == 1) then
					-- this condition checks to see what type of mount it is, which we already know.  If the filter is set to allow slow mounts through, then it is selected by default.  Otherwise, it's unselected by default
					LivestockSettings.Mounts[name].show = 1
				else
					LivestockSettings.Mounts[name].show = 0
				end
			end
			
		elseif name then -- if we do have SV data for it, update the index
			LivestockSettings.Mounts[name].index = i
		else
			mountfail = true
			--[[local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i)
			DEFAULT_CHAT_FRAME:AddMessage("Max mounts is " .. (GetNumCompanions("MOUNT") or "nil"))
			DEFAULT_CHAT_FRAME:AddMessage(string.format("Error for mount %u at redo companions, ID: %s, name: %s, spellID: %s, icon: %s, summoned: %s", i, (creatureID or "nil"), (creatureName or "nil"),(creatureSpellID or "nil"), (icon or "nil"), (issummoned or "nil")))]]
		end
		LivestockTooltip:Hide() -- hide our dummy tooltip
	end
	for k, v in pairs(LivestockSettings.Mounts) do
		if LivestockSettings.Mounts[k].type == "flying" then
			hasflying = true
			break
		end
	end
end

function Livestock.SmartPreClick(self) -- see if we're in an area that can be flown in.  If it is and we have a flying mount, set pickflying to be true.  Otherwise, set pickland to be true
	if InCombatLockdown() then -- if we're in combat, then get out of here because we can't set attributes in combat and want to avoid taint.
		return
	end
	
	local class = select(2, UnitClass("player"))
	
	if class == "DRUID" then -- druids need to cancel their current forms if they're in one in order to mount.  Scan the buffs to see if they're in a form.
		local buffs, name = 0
		repeat
			buffs = buffs + 1
			name = UnitBuff("player", buffs)
		until not name or name == L.LIVESTOCK_SPELL_BEARFORM or name == L.LIVESTOCK_SPELL_DIREBEARFORM or name == L.LIVESTOCK_SPELL_CATFORM or name == L.LIVESTOCK_SPELL_TRAVELFORM or name == L.LIVESTOCK_SPELL_TREEOFLIFE or name == L.LIVESTOCK_SPELL_MOONKINFORM or name == L.LIVESTOCK_SPELL_FLIGHTFORM or name == L.LIVESTOCK_SPELL_SWIFTFLIGHTFORM
		if name and not IsFlying() then -- only cancel forms if we're not flying, because otherwise we could cancel Flight Forms and we don't want that
			self:SetAttribute("type","spell")
			self:SetAttribute("spell",name)
		end
	else -- Non druids have to be on mounts to get anywhere (Ghost Wolf is dealt with later) so they all get the flag raised to mount in the PostClick.  Druids don't automatically raise the flag because otherwise we could end up casting Flight Form and then trying to mount.
		mountinpostclick = true
	end
	
	if GetSpellInfo(L.LIVESTOCK_SPELL_COLDWEATHERFLYING) and GetCurrentMapContinent() == 4 then -- if we're in Northrend and have the cold flying ability, then
		if (GetZoneText() == L.LIVESTOCK_ZONE_WINTERGRASP) then -- if we're in Wintergrasp, no flying allowed
			pickland = true
		elseif (GetZoneText() == L.LIVESTOCK_ZONE_DALARAN) then -- if we're in Dalaran, we might be able to fly
			if GetSubZoneText() == L.LIVESTOCK_SUBZONE_LANDING then -- if we're in Krasus's Landing, check to see if we're actually on the landing.  We can fly if we are, we can't if we're not.
				local x, y = GetPlayerMapPosition("player")
				if x >= 0.656 and x <= 0.78085 and y >= 0.35287 and y <= 0.563893 then
					pickflying = true
				else
					pickland = true
				end
			elseif GetSubZoneText() == L.LIVESTOCK_SUBZONE_UNDERBELLY then -- otherwise, see if we're in the Underbelly to see if we're in the exit pipe.  We can fly if we are, we can't if we're not
				local x, y = GetPlayerMapPosition("player")
				if x >= 0.092141166 and x <= 0.297127217055 and y >= 0.540120124816 and y <= 0.885304749012 then
					pickflying = true
				else
					pickland = true
				end
			else -- everywhere else in Dalaran, no flying
				pickland = true
			end
		else -- anywhere else in Northrend, we can fly!
			pickflying = true
		end
	elseif GetCurrentMapContinent() ~= 4 and IsFlyableArea() then -- if we're not in Northrend but the area is flyable, set the flying flag
		pickflying = true
	else -- set the land flag if all else fails
		pickland = true
	end  -- by this time, pickland or pickflying should be true, but not both.
	
	if LivestockSettings.druidlogic == 1 then -- if druid logic is on, then the button will become either a SFF, epic mount, or FF button
		if GetSpellInfo(L.LIVESTOCK_SPELL_SWIFTFLIGHTFORM) and pickflying then -- if the druid has Swift Flight Form and we can fly, then set the attributes to cast Swift Flight Form and get out of the function
			self:SetAttribute("type","spell")
			self:SetAttribute("spell",L.LIVESTOCK_SPELL_SWIFTFLIGHTFORM)
			return
		elseif pickflying then -- otherwise if we can fly, check to see what type of flying we'll be doing
			local mountcheck
			for k, v in pairs(LivestockSettings.Mounts) do
				if LivestockSettings.Mounts[k].type == "flying" and LivestockSettings.Mounts[k].speed == "epic" then -- if we find that there is an epic mount, we should prepare to mount it
					mountcheck = true -- set a flag if we have an epic mount, because we'll be using that one
					mountinpostclick = true -- set the flag to have the postclick handler call a mount
					break
				end
			end
			
			if mountcheck then -- if we set the flag already, we're done, so get out of the function
				return
			end
			
			if GetSpellInfo(L.LIVESTOCK_SPELL_FLIGHTFORM) then -- if mountcheck failed, there are no epic mounts, so we should cast normal Flight Form
				self:SetAttribute("type", "spell")
				self:SetAttribute("spell", L.LIVESTOCK_SPELL_FLIGHTFORM)
				return
			else -- if we've gotten this far, then we don't have any epic flying mounts and don't have flight form?  So mount a normal flying mount.
				mountinpostclick = true
				return
			end
			
		elseif not pickflying then -- if we're not in a flying area, pickland should be true so we can get out of the function after flagging to mount up
			mountinpostclick = true
			return
		end
	else -- no druid logic means deal with the mount as normal and get out of the function
		mountinpostclick = true
		return
	end
	
	if class == "SHAMAN" then -- if we're a shaman, we might be in GW form and we need to cancel this before summoning.
		local buff, name = 0
		repeat
			buff = buff + 1
			name = UnitBuff("player", buff)
		until not name or name == L.LIVESTOCK_SPELL_GHOSTWOLF
		
		if name then -- if name exists, it must be Ghost Wolf because the loop continues until name is nil or Ghost Wolf.  Set the button to dispel Ghost Wolf,
			self:SetAttribute("unit","player")
			self:SetAttribute("type","cancelaura")
			self:SetAttribute("spell", L.LIVESTOCK_SPELL_GHOSTWOLF)
			return
		end
	end
end

function Livestock.SmartPostClick(self)
	if mountinpostclick then -- we only summon mounts in the post click, and only if we were supposed to as defined in the preclick where mountinpostclick was set
		mountinpostclick = nil
		if pickland then
			Livestock.PickLandMount()
		else
			Livestock.PickFlyingMount()
		end
	end
	
	pickland, pickflying = nil, nil  --reset the button and the variables that decide if we're flying or not
	
	if not InCombatLockdown() then
		self:SetAttribute("type",nil)
	end
	
end

function Livestock.MakeCombatSmartMount(self, class)
	self:SetAttribute("type", "spell")
	
	if class == "DRUID" then
		self:SetAttribute("spell", L.LIVESTOCK_SPELL_TRAVELFORM)
	else
		self:SetAttribute("spell", L.LIVESTOCK_SPELL_GHOSTWOLF)
	end
end

-- Summon functions

function Livestock.PickCritter()
	local shown = 0
	
	Livestock.RecycleTable(temp)
	
	for k in pairs(LivestockSettings.Critters) do -- go throuh all the critters
		if LivestockSettings.Critters[k].show == 1 and not select(5, GetCompanionInfo("CRITTER",LivestockSettings.Critters[k].index)) then
			tinsert(temp, LivestockSettings.Critters[k].index) -- if a critter is marked as selected and it's NOT currently out in the game world, add its index to the temp table
		elseif LivestockSettings.Critters[k].show == 1 then
			shown = shown + 1 -- count the total number of critters marked as selected
		end
	end
	
	if #temp == 0 and shown ~= 1 then -- if there are no critters in the list AND the number of critters selected isn't 1, then there must be 0 critters selected.  Prompt user to select some.
		p(L.LIVESTOCK_INTERFACE_NOCRITTERSCHECKED)
	elseif #temp ~= 0 then -- otherwise, there are some critters in the list.
		currentcritter = temp[random(#temp)] -- set the newest critter to be one of the random indices among those in the temp table.
		CallCompanion("CRITTER",currentcritter) -- call the critter
	end
end

function Livestock.DismissCritter()
	if currentcritter then -- if we happen to know what the current critter is (that is, it was called with PickCritter()) then we can dismiss it without looping through all of them
		CallCompanion("CRITTER", currentcritter)
		return
	end
	
	for i = 1, GetNumCompanions("CRITTER") do -- loop through all the critters, and when we find the index whose critter is active, summon it again (which desummons it) and stop looping
		if select(5, GetCompanionInfo("CRITTER", i)) then
			CallCompanion("CRITTER", i)
			break
		end
	end
end

function Livestock.PickLandMount()

	if IsMounted() and not IsFlying() then -- if we're already mounted and NOT flying (no accidents, please!) then dismount and don't pick another mount
		Dismount()
		return
	end
	
	if UnitInVehicle("player") and not IsFlying() then
		VehicleExit()
		return
	end

	Livestock.RecycleTable(temp)

	for k in pairs(LivestockSettings.Mounts) do -- go through the land mounts and add the ones that are selected to the temp table
		if LivestockSettings.Mounts[k].type == "land" and LivestockSettings.Mounts[k].show == 1 then
			tinsert(temp, LivestockSettings.Mounts[k].index)
		end
	end
	
	if #temp == 0 then -- if no mounts are selected, prompt the user to select one
		p(L.LIVESTOCK_INTERFACE_NOLANDMOUNTSCHECKED)
		return
	end
	
	CallCompanion("MOUNT",temp[random(#temp)]) -- call a random mount
end

function Livestock.PickFlyingMount()

	if not hasflying then
		Livestock.PickLandMount()
		return
	end

	if IsFlying() then -- if we're already flying, don't do anything; if we're not and mounted, we're on the ground on a flying mount so dismount instead of summoning another mount
		return
	elseif IsMounted() then
		Dismount()
		return
	elseif UnitInVehicle("player") then
		VehicleExit()
		return
	end
	
	Livestock.RecycleTable(temp)

	for k in pairs(LivestockSettings.Mounts) do -- go through the flying mounts and add the ones that are selected to the temp table
		if LivestockSettings.Mounts[k].type == "flying" and LivestockSettings.Mounts[k].show == 1 then
			tinsert(temp, LivestockSettings.Mounts[k].index)
		end
	end
	
	if #temp == 0 then -- if no mounts are selected, prompt the user to select one
		p(L.LIVESTOCK_INTERFACE_NOFLYINGMOUNTSCHECKED)
		return
	end
	
	CallCompanion("MOUNT",temp[random(#temp)]) -- call a random mount
end

-- Slash handler

function Livestock.Slash(arg)
	if arg == "druid" then
		if select(2, UnitClass("player")) ~= "DRUID" then
			p(L.LIVESTOCK_INTERFACE_NOTADRUIDERROR)
			return
		else
			p(string.format(L.LIVESTOCK_INTERFACE_DRUIDTOGGLEMESSAGE, (LivestockSettings.druidlogic == 1 and L.LIVESTOCK_INTERFACE_NO) or L.LIVESTOCK_INTERFACE_YES))
			if LivestockMacroFrameToggleDruidLogic:GetChecked() then
				LivestockMacroFrameToggleDruidLogic:SetChecked(false)
			else
				LivestockMacroFrameToggleDruidLogic:SetChecked(true)
			end
			LivestockSettings.druidlogic = 1 - LivestockSettings.druidlogic
			return
		end
	elseif arg == "reset" then -- Clear all the points of the Livestock buttons and set them back to their original locations
		LivestockCrittersButton:ClearAllPoints()
		LivestockCrittersButton:SetPoint("CENTER",-50,-150)
		LivestockLandMountsButton:ClearAllPoints()
		LivestockLandMountsButton:SetPoint("CENTER",0,-150)
		LivestockFlyingMountsButton:ClearAllPoints()
		LivestockFlyingMountsButton:SetPoint("CENTER",50,-150)
		LivestockSmartButton:ClearAllPoints()
		LivestockSmartButton:SetPoint("CENTER",0,-190)
		return
	elseif arg:sub(1,5) == "scale" then -- clear all the points of the Livestock buttons, set them back to their original locations, and then set the scale
		LivestockCrittersButton:ClearAllPoints()
		LivestockCrittersButton:SetPoint("CENTER",-50,-150)
		LivestockLandMountsButton:ClearAllPoints()
		LivestockLandMountsButton:SetPoint("CENTER",0,-150)
		LivestockFlyingMountsButton:ClearAllPoints()
		LivestockFlyingMountsButton:SetPoint("CENTER",50,-150)
		LivestockSmartButton:ClearAllPoints()
		LivestockSmartButton:SetPoint("CENTER",0,-190)
		Livestock.ScaleButtons(tonumber(arg:sub(7,13)))
		return
	elseif arg == "auto" then
		p(string.format(L.LIVESTOCK_INTERFACE_AUTOREZTOGGLEMESSAGE, (LivestockSettings.autosummon == 1 and L.LIVESTOCK_INTERFACE_NO) or L.LIVESTOCK_INTERFACE_YES))
			if LivestockMacroFrameToggleAutosummon:GetChecked() then
				LivestockMacroFrameToggleAutosummon:SetChecked(false)
			else
				LivestockMacroFrameToggleAutosummon:SetChecked(true)
			end
			LivestockSettings.autosummon = 1 - LivestockSettings.autosummon
			return
	elseif arg == "help" or arg == "about" then
		p("|cFF4444DD Livestock, version "..LIVESTOCK_VERSION..". Available slash commands:")
		p("|cFF4444DDLivestock:|r '/livestock' toggles the Livestock Menu.")
		p("|cFF4444DDLivestock:|r '/livestock scale <scale>' sets the scale of the Livestock buttons to <scale>.")
		p("|cFF4444DDLivestock:|r '/livestock reset' places all Livestock buttons back at their default locations.")
		p("|cFF4444DDLivestock:|r '/livestock druid' toggles using Flight forms in Smart Mounting (druids only).")
		p("|cFF4444DDLivestock:|r '/livestock auto' toggles automatically summoning a vanity pet when you return to life.")
	else -- toggle the Livestock menu frame
		if LivestockMenuFrame:IsVisible() then
			LivestockMenuFrame:Hide()
		else
			LivestockMenuFrame:Show()
		end
	end
end

-- Companion Frame declaration and slash handler setup
local companionframe = CreateFrame("Frame")
companionframe:RegisterEvent("COMPANION_LEARNED")
companionframe:RegisterEvent("PLAYER_LOGIN")
companionframe:RegisterEvent("ZONE_CHANGED")
companionframe:RegisterEvent("ZONE_CHANGED_INDOORS")
companionframe:RegisterEvent("ZONE_CHANGED_NEW_AREA")
companionframe:RegisterEvent("PLAYER_REGEN_DISABLED")
companionframe:RegisterEvent("PLAYER_UNGHOST")
companionframe:RegisterEvent("PLAYER_ALIVE")
companionframe:RegisterEvent("PLAYER_ENTERING_WORLD")
companionframe:SetScript("OnEvent", Livestock.CompanionEvent)

SLASH_LIVESTOCK1 = LIVESTOCK_INTERFACE_SLASHSTRING
SlashCmdList["LIVESTOCK"] = Livestock.Slash